<?

/* This file is part of "DAV Drupal Module".
 *    Copyright 2008, karma-lab.net
 *    Author : Yoran Brault
 *    eMail  : software@_bad_karma-lab.net (remove _bad_ before sending an email)
 *    Site   : http://artisan.karma-lab.net
 *
 * "DAV Drupal Module" is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * "DAV Drupal Module" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with "DAV Drupal Module"; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */


/**
 * Resolve a given arguments list according to a specific resource.
 *
 * @param $arguments resource arguments
 * @param $path resource path
 * @return If resource is resoled, an array of wildcards=>according argument; false else.
 */
function _webdav_resolve_resource_path($arguments, $path)  {
  $route=explode("/",_webdav_unslashify($path));
  if (strlen($route[0])==0) {
    unset($route[0]);
  }

  if (count($route)==count($arguments)) {
    $result=array();
    $match=true;
    for ($i=0; $i < count($arguments); $i++) {
      if ($route[$i]{0}=='%') {
        $result[$route[$i]]=$arguments[$i];
      } else {
        if ($route[$i]!=$arguments[$i]) {
          $match=false;
          break;
        }
      }
    }
    if ($match) {
      return $result;
    }
  }
  return false;
}


/**
 * Try to resolve resource arguments with known resources as given by hook_webdav_resources.
 *
 * @param $arguments resource arguments
 * @return if a route is found, an array containing resource path wildcards values (variable arguments) and
 * resource id in an associating array, else false.
 */
function webdav_resolve_resource($arguments) {

  global $webdav_resources;
  if (empty($webdav_resources)) {
    $webdav_resources=module_invoke_all('webdav_resources');
    if (webdav_is_debug()) webdav_debug(count($webdav_resources)." resources found.");
    // FIXME we should now if there is resource conflict.
  }
  foreach ($webdav_resources as $path=>$resource) {
    $match = _webdav_resolve_resource_path($arguments, $path);
    if ($match !== false) {
      $result['id']=$resource['id'];
      $result['module']=$resource['module'];
      $result['parameters']=$match;
      $result['arguments']=$arguments;
      foreach ($result['parameters'] as $key=>$value) {
        if ($key{0}=='%') {
          unset($result['parameters'][$key]);
          $key=substr($key,1);
          $loader=$resource['module'].'_'.$key."_load";
          if (function_exists($loader)) {
            $loader_result = $loader($value);
            if ($loader_result===false) {
              webdav_debug("Error while loading '".$value."' with ".$loader);
              return false;
            } else {
              $value=$loader_result;
            }
          }
          $result['parameters'][$key]=$value;
        }
      }
      if (webdav_is_debug()) {
        webdav_debug("Found a matching resource '".$resource['id']."' with '".$path."' path.");
        foreach ($result['parameters'] as $key=>$value) {
          webdav_debug("  * ".$key." => ".$value." (".gettype($value).")");
        }
      }
      return (object)$result;
    }
  }
  if (webdav_is_debug()) {
    webdav_debug("No resource match !!");
  }
  return false;
}