<?php

/*
 * Copyright 2005, 2006 by Yoran Brault (yoran@karma-lab.net)
 * http://artisan.karma-lab.net).
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

require_once 'HTTP/WebDAV/Server.php';


class DrupalDavServer extends HTTP_WebDAV_Server {
  var $dav_powered_by = 'Drupal WebDAV (+http://drupal.org/project/webdav)';

  /** Serve a webdav request. */
  function ServeRequest() {
    $site_name=variable_get('site_name', null);
    if ($site_name) {
      $site_name=" for '".$site_name."'";
    }
    $this->http_auth_realm = "Drupal WebDav Access".$site_name;
    parent :: ServeRequest();
  }

  /** authentication needed here. */
  function check_auth($type, $username, $password) {
    // fast track to display authentication dialog on client side
    if (strlen($username)==0) {
      if (dav_is_debug) webdav_debug("   Need authentication, bye...");
      return false;
    }

    $user= user_authenticate(array('name'=>$username, 'pass'=>$password));
    if (user_access(DAV_PERMS_WEBDAV_ACCESS, $user) == TRUE) {
      if (dav_is_debug) webdav_debug("Login succeed for ".$username);
      return true;
    }
    if (dav_is_debug) webdav_debug("Login failed for ".$username);
    return false;
  }

  function checkLock ($path) {
    //    $cursor = db_query("SELECT * FROM {webdav_locks} WHERE path = '%s'",$path);
    //    $lock = db_fetch_array($cursor);
    //    if (webdav_is_debug()) webdav_debug("Checking $path lock... : ".$lock['token']);
    //    return $lock;
    return NULL;
  }

  /** PROPFIND method handler. */
  function PROPFIND(& $options, & $files) {
    error_log($options['path']);
    $session=webdav_start_session($options);
    if (webdav_is_error($session)) return webdav_close_session($session);

    $items=array();

    // search for the associated resource
    $resource   = webdav_resolve_resource($session->arguments);
    $root_item  = webdav_lookup($resource);
    if ($root_item===false) {
      //return webdav_close_session($session, WEBDAV_STATUS_NOT_FOUND);
      return false;
    }

    // if client forget the leading / for a collection URL, we should redirect it
    webdav_redirect_wrong_collection($item);

    /*  first, this is a folder...
     *  FIXME sure this will not work for depth > 1
     */
    if ($root_item['type'] == 'folder' && $options['depth'] != 0) {
      $items = webdav_enumerate_items($resource);
    }

    // make the root item first of the list
    array_unshift($items, $root_item);

    // Convert items to DAV files
    $files['files'] = array ();
    $items[0]['id']=''; //
    foreach ($items as $item) {
      $files['files'][] = $this->convert_item($session, $item);
    }

    // DEBUG: display result
    if (webdav_is_debug()) {
      $count=0;
      foreach ($files['files'] as $file) {
        webdav_debug('   - ' . $file['path']);
        $count++;
        if ($count > 5) {
          break;
        }
      }
    }

    webdav_close_session($session);
    return true; // No status for this, everything is handled by undelying class
  }

  /** convert DFS item to Dav item */
  function convert_item(&$session, &$item) {
    $file = array ();

    // Common properties
    $file['path'] = _webdav_encode_path($session->path . "/" . $item['id']);
    $file['props'] = array ();

    if (!empty($item['name'])) {
      $file['props'][] = $this->mkprop(
        'displayname',$item['name']); // No need to convert HTML entities (< > &) as is done by pear
    }
     
    if (!empty($item['created'])) {
      $file['props'][] = $this->mkprop('creationdate', $item['created']);
    }
    if (!empty($item['changed'])) {
      $file['props'][] = $this->mkprop('getlastmodified', $item['changed']);
    }

    if ($item['type'] == 'folder') {
      $file['path'] = _webdav_slashify($file['path']);
      $file['props'][] = $this->mkprop('resourcetype', 'collection');
      $file['props'][] = $this->mkprop('getcontentlength', 1);
    } else {
      $file['path'] = _webdav_unslashify($file['path']);
      $file['props'][] = $this->mkprop('creationdate', $item['created']);
      $file['props'][] = $this->mkprop('getlastmodified', $item['changed']);
      $file['props'][] = $this->mkprop('resourcetype', '');
      $file['props'][] = $this->mkprop('getcontenttype', $item['type']);
      $file['props'][] = $this->mkprop('getcontentlength', $item['size']);
    }
    return $file;
  }

  /**
   * GET method handler
   *
   * @param  array  parameter passing array
   * @return bool   true on success
   */
  function GET(& $options) {
    $session=webdav_start_session($options);
    if (webdav_is_error($session)) return webdav_close_session($session);

    // search for the associated resource
    $resource   = webdav_resolve_resource($session->arguments);
    $item  = webdav_lookup($resource);
    if ($item===false) {
      return webdav_close_session($session, WEBDAV_STATUS_NOT_FOUND);
    }

    // if client forget the leading / for a collection URL, we should redirect it
    webdav_redirect_wrong_collection($item);

    if ($item['type']!='folder') {
      if (!$item['encoding']) {
        $item['encoding']="UTF8";
      }
      $options['mimetype'] = $item['type'].'; charset=' . $item['encoding'];
      $options['mtime'] = $item['changed'];
      $options['size'] = $item['size'];

      // Load string content
      webdav_invoke_resource($session, $resource, 'webdav_get');
      if (webdav_is_error($session)) return webdav_close_session($session);

      if (!empty($session->content_string)) {
        error_log("STRING:".$session->content_string);
        // Save temporary file
        $file_name = tempnam('/tmp', 'FOO');
        $handle = fopen($file_name, 'w');
        fwrite($handle, $session->content_string);
        fclose($handle);
        $stream=fopen($file_name, 'r');
      } else {
        $stream=$session->content_stream;
      }
      $options['stream'] = $stream;
      webdav_close_session($session);
      return true;
    } else {
      $items = webdav_enumerate_items($resource);
      foreach ($items as &$item) {
        if (empty($item['name'])) {
          $item['name']=$item['id'];
        }
      }
      print theme('webdav_listing', $item, $items);
      header($session->status);
      webdav_close_session($session);
      exit();
    }
  }

  /**
   * PUT method handler
   *
   * @param  array  parameter passing array
   * @return bool   true on success
   */
  function PUT(& $options) {
    $session=webdav_start_session($options);
    if (webdav_is_error($session)) return webdav_close_session($session);

    // lookup item (if it is an existing one)
    $arguments=$session->arguments;
    $name=array_pop($arguments);
    $parent_resource   = webdav_resolve_resource($arguments);
    $parent_item  = webdav_lookup($parent_resource);
    if ($parent_item===false) {
      return webdav_close_session($session, WEBDAV_STATUS_NOT_FOUND);
    }

    // prepare item stuff
    $file_name = file_directory_temp()."/".$name;
    $handle = fopen($file_name, 'w');
    while (!feof($options["stream"])) {
      $data= fread($options["stream"], 8192);
      fwrite($handle, $data);
    }
    fclose($handle);

    webdav_invoke_resource($session, $parent_resource, 'webdav_put', $file_name);
    if ($session->status==WEBDAV_STATUS_OK) $session->status=WEBDAV_STATUS_CREATED;
    return webdav_close_session($session);
  }

  /**
   * MKCOL method handler
   *
   * @param  array  general parameter passing array
   * @return bool   true on success
   */
  function MKCOL($options) {
    $session=webdav_start_session($options);
    if (webdav_is_error($session)) return webdav_close_session($session);
    $session->status=WEBDAV_STATUS_CREATED;

    // lookup item
    $new_folder=array_pop($session->arguments);
    $resource   = webdav_resolve_resource($session->arguments);
    $item  = webdav_lookup($resource);
    if ($item===false) {
      return webdav_close_session($session, WEBDAV_STATUS_NOT_FOUND);
    }

    webdav_invoke_resource($session, $resource, 'webdav_create', $new_folder);
    if ($session->status==WEBDAV_STATUS_OK) $session->status=WEBDAV_STATUS_CREATED;
    return webdav_close_session($session);
  }

  /**
   * DELETE method handler
   *
   * @param  array  general parameter passing array
   * @return bool   true on success
   */
  function DELETE($options) {
    $session=webdav_start_session($options);
    if (webdav_is_error($session)) return webdav_close_session($session);

    // lookup item
    $resource   = webdav_resolve_resource($session->arguments);
    $item  = webdav_lookup($resource);
    if ($item===false) {
      return webdav_close_session($session, WEBDAV_STATUS_NOT_FOUND);
    }


    // prepare item stuff
    webdav_invoke_resource($session,$resource, 'webdav_delete');
    if ($session->status==WEBDAV_STATUS_OK) {
      $session->status=WEBDAV_STATUS_NO_CONTENT;
      // cleaning locks
      db_query("DELETE FROM {webdav_locks} WHERE path='%s'", $options['path']);
    }
    return webdav_close_session($session);
  }

  /**
   * OPTIONS method handler
   *
   * @param  array  general parameter passing array
   * @return bool   true on success
   */
  function OPTIONS($options) {
    $session=webdav_start_session($options);
    if (webdav_is_error($session)) return webdav_close_session($session);

    // lookup item
    $resource   = webdav_resolve_resource($session->arguments);
    $item  = webdav_lookup($resource);
    if ($item===false) {
      return webdav_close_session($session, WEBDAV_STATUS_NOT_FOUND);
    }

    // if client forget the leading / for a collection URL, we should redirect it
    webdav_redirect_wrong_collection($item);

    $item=webdav_lookup($resource);
    header("Content-Type:".$item['type']=='folder'?'httpd/unix-directory':$item['type']);
    webdav_close_session($session, WEBDAV_STATUS_OK); // We
  }


  /**
   * MOVE method handler
   *
   * @param  array  general parameter passing array
   * @return bool   true on success
   */
  function MOVE($options) {
    $session=webdav_start_session($options);
    if (webdav_is_error($session)) return webdav_close_session($session);

    // lookup item
    $arguments=$session->arguments;
    $source_resource   = webdav_resolve_resource($arguments);
    $source_item  = webdav_lookup($source_resource);
    if ($source_item===false) {
      return webdav_close_session($session, WEBDAV_STATUS_NOT_FOUND);
    }

    $target_session=webdav_create_session($options['dest']);
    $target_name=array_pop($target_session->arguments);
    $target_resource   = webdav_resolve_resource($target_session->arguments);
    $target_item  = webdav_lookup($target_resource);
    if ($target_item===false) {
      return webdav_close_session($session, WEBDAV_STATUS_NOT_FOUND);
    }

    webdav_invoke_resource(
    $session,
    $target_resource, 'webdav_move',
    $target_name,
    $source_resource);
    if ($session->status==WEBDAV_STATUS_OK) {
      $session->status=WEBDAV_STATUS_CREATED;
      // moving locks
      db_query("update {webdav_locks} set path='%s' where path='%s'",$options['dest'],$options['path']);
    }
    return webdav_close_session($session);
  }


  /**
   * COPY method handler
   *
   * @param  array  general parameter passing array
   * @return bool   true on success
   */
  function COPY($options) {
    $session=webdav_start_session($options);
    if (webdav_is_error($session)) return webdav_close_session($session);

    // lookup item
    $arguments=$session->arguments;
    $source_resource   = webdav_resolve_resource($arguments);
    $source_item  = webdav_lookup($source_resource);
    if ($source_item===false) {
      return webdav_close_session($session, WEBDAV_STATUS_NOT_FOUND);
    }

    $target_session=webdav_create_session($options['dest']);
    $target_name=array_pop($target_session->arguments);
    $target_resource   = webdav_resolve_resource($target_session->arguments);
    $target_item  = webdav_lookup($target_resource);
    if ($target_item===false) {
      return webdav_close_session($session, WEBDAV_STATUS_NOT_FOUND);
    }

    webdav_invoke_resource(
    $session,
    $target_resource, 'webdav_copy',
    $target_name,
    $source_resource);
    if ($session->status==WEBDAV_STATUS_OK) $session->status=WEBDAV_STATUS_CREATED;
    return webdav_close_session($session);
  }

  /**
   * PROPPATCH method handler
   *
   * @param  array  general parameter passing array
   * @return bool   true on success
   */
  function PROPPATCH(& $options) {
    $session=webdav_start_session($options);
    webdav_status_not_implemented();
    if (webdav_is_error($session)) return webdav_close_session($session);
  }

  /**
   * LOCK method handler
   *
   * @param  array  general parameter passing array
   * @return bool   true on success
   */
  function LOCK(& $options) {
    $session=webdav_start_session($options);
    if (webdav_is_error($session)) return $session->status;

    // lookup the resource to lock
    $resource   = webdav_resolve_resource($session->arguments);
    if ($resource===false) {
      return webdav_close_session($session, WEBDAV_STATUS_NOT_FOUND);
    }

    webdav_invoke_resource($session, $resource, 'webdav_lock');
    if (webdav_is_error($session)) {
      return webdav_close_session($session);
    }

    $options["timeout"] = time()+webdav_lock_timeout();

    if (isset($options["update"])) { // Lock Update
      $cursor = db_query("SELECT * FROM {webdav_locks} WHERE path = '%s' AND token = '%s'",
      $options[path],
      $options[update]);
       
      $lock = db_fetch_array($cursor);
      if ($lock) {
        $lock['expires']=$options[timeout];
        $lock['modified']=time();
        drupal_write_record('dav_locks', $lock, 'token');
        $options['owner'] = $lock['owner'];
        $options['scope'] = $lock["exclusivelock"] ? "exclusive" : "shared";
        $options['type']  = $lock["exclusivelock"] ? "write"     : "read";
        return true;
      } else {
        return false;
      }
    }

    $lock=array(
    	'token'		=> 		$options[locktoken],
    	'path'		=> 			$options[path],
			'created' => 	time(),
			'modified' => time(),
      'owner'   => $options[owner],
			'expires' => $options[timeout],
			'exclusivelock'  => $options['scope'] === "exclusive" ? "1" : "0",
    );
    drupal_write_record('webdav_locks', $lock);

    return webdav_close_session($session);
  }

  /**
   * UNLOCK method handler
   *
   * @param  array  general parameter passing array
   * @return bool   true on success
   */
  function UNLOCK(& $options) {
    $session=webdav_start_session($options);
    if (webdav_is_error($session)) return $session->status;

    // lookup the resource to lock
    $resource   = webdav_resolve_resource($session->arguments);
    if ($resource===false) {
      return webdav_close_session($session, WEBDAV_STATUS_NOT_FOUND);
    }

    webdav_invoke_resource($session, $resource, 'webdav_lock');
    if (webdav_is_error($session)) {
      return webdav_close_session($session);
    }

    db_query("DELETE FROM {webdav_locks} where path='%s' and token='%s'",
    $options[path],
    $options[token]);
    return webdav_close_session($session, WEBDAV_STATUS_NO_CONTENT);
  }
}
