<?

/* This file is part of "DAV Drupal Module".
 *    Copyright 2008, karma-lab.net
 *    Author : Yoran Brault
 *    eMail  : software@_bad_karma-lab.net (remove _bad_ before sending an email)
 *    Site   : http://artisan.karma-lab.net
 *
 * "DAV Drupal Module" is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * "DAV Drupal Module" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with "DAV Drupal Module"; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

define('WEBDAV_STATUS_OK',"200 OK");
define('WEBDAV_STATUS_CREATED',"201 Created");
define('WEBDAV_STATUS_NO_CONTENT',"204 No Content");
define('WEBDAV_STATUS_NOT_FOUND', "404 Not Found");
define('WEBDAV_STATUS_FORBIDDEN', "403 Forbidden");
define('WEBDAV_STATUS_INTERNAL_SERVER_ERROR',"500 Internal Server Error");
define('WEBDAV_STATUS_NOT_IMPLEMENTED',"501 Not Implemented");
define('WEBDAV_STATUS_CONFLICT',"409 Conflict");

/**
 * Session is basicaly a wrapper arround
 * and exploded path called as 'arguments'
 *
 * @param $path ressource path
 * @return session array
 */
function webdav_create_session($path) {
  // Explode path in argument array
  $path=_webdav_unslashify($path);
  $arguments=explode("/", $path);

  // Remove the first one (drupal menu item, namely 'webdav')
  array_shift($arguments);
  for ($i=0; $i<count($arguments); $i++) {
    $arguments[$i]=rawurldecode($arguments[$i]);
  }

  // build the context
  return (object)array(
    'arguments'=>$arguments,
    'path'=>$path,
    'error'=>false,
  );
}

/**
 * Should be first call by any DAV server method.
 *
 * @param $options DAV header options
 *
 */
function webdav_start_session($options) {
  $session=webdav_create_session($options['path']);
  if (!user_access(DAV_PERMS_WEBDAV_ACCESS)) {
    webdav_status_forbidden($session);
  }
  return $session;
}


function webdav_close_session(&$session, $last_result=NULL) {
  if ($last_result!=NULL) {
    $session->status=$last_result;
  }
  webdav_debug_close_query($session);
  if (empty($session->status)) {
    return true;
  }
  return $session->status;
}

function webdav_is_error(&$session) {
  if (empty($session->status)) {
    return false;
  }
  return $session->status{0}>2;
}

function webdav_result(&$session, $status) {
  $session->status=$status;
}
